RobPower <- function(A,D,h2,threshold,var.QTL,methods,GC,r2){

n = nrow(A)
mu=0
vary = 1


#---- 0) Errors control
source(file="fonctions/controle.erreur.r")
cpt.break = controle.erreur(h2,methods,var.QTL,GC)
if(cpt.break==0) { # no error
	# add 0 for robustness computing
	tmpvQTL = var.QTL
	var.QTL = c(0,as.numeric(var.QTL))*r2

	#---- 1) Loading source
	print(paste("Loading source",sep=""),quote=FALSE)
	source(file="fonctions/calc.xprimx.r")
	source(file="fonctions/calc.xprimAx.r")
	source(file="fonctions/regression.r")
	source(file="fonctions/calc.qprimq.r")
	source(file="fonctions/calc.qprimx.r")
	source(file="fonctions/calc.qprimAq.r")
	source(file="fonctions/calc.sseqtdt.r")
	source(file="fonctions/qtdt.r")
	source(file="fonctions/calc.invMME.r")
	source(file="fonctions/calc.XprimCuuX.r")
	source(file="fonctions/calc.XprimCuuCuuX.r")
	source(file="fonctions/calc.XprimCuuv1CuuX.r")
	source(file="fonctions/calc.ssegrammar.r")
	source(file="fonctions/grammar.r")
	source(file="fonctions/calc.ssefasta.r")
	source(file="fonctions/fasta.r")

	A=as.matrix(A)
	D=as.matrix(D)

	xprimx = calc.xprimx(A)
	xprimAx = calc.xprimAx(A)

	Resultat <- list()

	#-- Regression ---
	met.reg=length(which(methods=="reg"))
	if(met.reg>0) {
		print(paste("Computing robustness and power for regression model",sep=""),quote=FALSE)
		Result.reg = regression(A,n,threshold,h2,var.QTL,vary,xprimx,xprimAx,GC,tmpvQTL)
		if(GC==TRUE) {
			names(Result.reg) <- c("Robustness","power","GC")
			Resultat[[1]]=Result.reg
		} else {
			names(Result.reg) <- c("Robustness","power")
			Resultat[[1]]=Result.reg
		}
	} else {
		Resultat[[1]]=999
	}

	#-- QTDT ---
	met.qtdt=length(which(methods=="qtdt"))
	if(met.qtdt>0) {	
		QprimQ = calc.qprimq(A,D)
		Qprimx = calc.qprimx(A,D)
		Qprim1 = c(n,0,0)
		QprimAQ = calc.qprimAq(A,D)
		SSEqtdt = calc.sseqtdt(h2,var.QTL,A)
		SSEqtdt.rob = calc.sseqtdt(h2,0,A)
		print(paste("Computing robustness and power for QTDT method",sep=""),quote=FALSE)
		Result.qtdt = qtdt(A,n,threshold,h2,var.QTL,mu,QprimQ,Qprimx,Qprim1,QprimAQ,SSEqtdt,SSEqtdt.rob,tmpvQTL)
		names(Result.qtdt) <- c("Robustness","power")
		Resultat[[2]]=Result.qtdt
	} else {
		Resultat[[2]]=999
	}

	# Solving MME for GRAMMAR and/or FASTA
	test.MME.grammar = length(which(methods=="grammar"))
	test.MME.fasta = length(which(methods=="fasta"))
	if(test.MME.grammar>0 | test.MME.fasta>0) {
		beta = sqrt(var.QTL)
		h2new=matrix(0,length(h2),length(beta))
		lambda=matrix(0,length(h2),length(beta))
		for(i in 1:length(beta)) {
			h2new[,i] = (h2+(beta[i]*beta[i]))/(1+(beta[i]*beta[i]))
			lambda[,i] = (1-h2new[,i])/(h2new[,i])
		}
		invMME = calc.invMME(A,h2,lambda,var.QTL)
		C11.h2 <- list()
		C1u.h2 <- list()
		Cu1.h2 <- list()
		Cuu.h2 <- list()
		for(i in 1:length(h2)) {
			C11.beta <- list()
			C1u.beta <- list()
			Cu1.beta <- list()
			Cuu.beta <- list()
			for(j in 1:length(var.QTL)) {
				C11.beta[[j]] = invMME[[i]][[j]][1,1]
				C1u.beta[[j]] = invMME[[i]][[j]][1,2:ncol(invMME[[i]][[j]])]
				Cu1.beta[[j]] = invMME[[i]][[j]][2:nrow(invMME[[i]][[j]]),1]
				Cuu.beta[[j]] = invMME[[i]][[j]][2:nrow(invMME[[i]][[j]]),2:ncol(invMME[[i]][[j]])]
			}
			C11.h2[[i]] = C11.beta
			C1u.h2[[i]] = C1u.beta
			Cu1.h2[[i]] = Cu1.beta
			Cuu.h2[[i]] = Cuu.beta
			# Chacun des Cxx.h2[i] contient les Cxx pour l'heritabilite i et tous les beta possible
		}
		XprimCuuX = calc.XprimCuuX(A,h2,lambda,Cuu.h2)
		XprimCuuCuuX = calc.XprimCuuCuuX(A,h2,lambda,Cuu.h2)
		XprimCuuv1CuuX = calc.XprimCuuv1CuuX(A,h2,lambda,Cuu.h2)
	}
	
	#-- GRAMMAR ---
	met.grammar=length(which(methods=="grammar"))
	if(met.grammar>0) {	
		SSEgrammar = calc.ssegrammar(A,h2,lambda,var.QTL,XprimCuuX,XprimCuuCuuX,XprimCuuv1CuuX,xprimx,xprimAx,Cuu.h2)
		print(paste("Computing robustness and power for GRAMMAR method",sep=""),quote=FALSE)
		Result.grammar = grammar(A,n,threshold,h2,lambda,var.QTL,mu,xprimx,XprimCuuX,XprimCuuCuuX,XprimCuuv1CuuX,SSEgrammar,GC,tmpvQTL)
		if(GC==TRUE) {
			names(Result.grammar) <- c("Robustness","power","GC")
			Resultat[[3]]=Result.grammar
		} else {
			names(Result.grammar) <- c("Robustness","power")
			Resultat[[3]]=Result.grammar
		}
	} else {
		Resultat[[3]]=999
	}

	#-- FASTA ---
	met.fasta=length(which(methods=="fasta"))
	if(met.fasta>0) {	
		SSEfasta = calc.ssefasta(A,h2,lambda,var.QTL,XprimCuuX,XprimCuuCuuX,XprimCuuv1CuuX,xprimx,xprimAx,Cuu.h2)
		print(paste("Computing robustness and power for FASTA method",sep=""),quote=FALSE)
		Result.fasta = fasta(A,n,threshold,h2,lambda,var.QTL,mu,xprimx,XprimCuuX,XprimCuuCuuX,XprimCuuv1CuuX,SSEfasta,tmpvQTL)
		names(Result.fasta) <- c("Robustness","power")
		Resultat[[4]]=Result.fasta
	} else {
		Resultat[[4]]=999
	}

	#-- Results output
	names(Resultat) <- c("regression","qtdt","grammar","fasta")
	res=Resultat[Resultat!="999"]
	return(res)

} else { # error detected
	print(paste("Error : END OF PROGRAM",sep=""),quote=FALSE)
}

}